///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 1995 - 2004 OSR Open Systems Resources, Inc.
//    All Rights Reserved
//
//    This sofware is supplied for instructional purposes only.
//
//    OSR Open Systems Resources, Inc. (OSR) expressly disclaims any warranty
//    for this software.  THIS SOFTWARE IS PROVIDED  "AS IS" WITHOUT WARRANTY
//    OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION,
//    THE IMPLIED WARRANTIES OF MECHANTABILITY OR FITNESS FOR A PARTICULAR
//    PURPOSE.  THE ENTIRE RISK ARISING FROM THE USE OF THIS SOFTWARE REMAINS
//    WITH YOU.  OSR's entire liability and your exclusive remedy shall not
//    exceed the price paid for this material.  In no event shall OSR or its
//    suppliers be liable for any damages whatsoever (including, without
//    limitation, damages for loss of business profit, business interruption,
//    loss of business information, or any other pecuniary loss) arising out
//    of the use or inability to use this software, even if OSR has been
//    advised of the possibility of such damages.  Because some states/
//    jurisdictions do not allow the exclusion or limitation of liability for
//    consequential or incidental damages, the above limitation may not apply
//    to you.
//
//    OSR Open Systems Resources, Inc.
//    105 Route 101A Suite 19
//    Amherst, NH 03031  (603) 595-6500 FAX: (603) 595-6503
//    email bugs to: bugs@osr.com
//
//
//    MODULE:
//
//        iv2.cpp - Increment Version code
//
//    ABSTRACT:
//
//      Increment Version Code
//
//    AUTHOR(S):
//
//        OSR Open Systems Resources, Inc.
// 
//    REVISION:   
//
///////////////////////////////////////////////////////////////////////////////
// iv2.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "iv2.h"
#include <io.h>
#include <SYS\STAT.H>


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

//using namespace std;

#define PLATFORM_MAX_SIZE 56

char * filename = "version.h";
extern int program_build_number;
int version = 0;
int report=0;
int noIncrement = 0;
int override = 0;
int major = 0;
int minor = 0;
int baselevel = 0;
int generation = 0;
char platform[PLATFORM_MAX_SIZE];
char * buildType = "";

int usage(char * name, char * msg, int errno)
{
    if (msg) {
        fprintf(stderr, "%s\n", msg);
    }
    fprintf(stderr, "%s (version 2 build %d)\n Usage:[-r][-h][-n][-a] [-b buildType] "
                    "[-f headerfile] [-v version] fileList\n", name, program_build_number);

    fprintf(stderr, "     -r  report on everything done to stdout\n\n");

    fprintf(stderr, "     -n  no increment, just print the current version number on stdout\n\n");

    fprintf(stderr, "     -h  help (this is it.)\n\n");

    fprintf(stderr, "     -f  headerfile indicates the file to modify.\n");
    fprintf(stderr, "         The default filename is version.h\n\n");

    fprintf(stderr, "     -b  indicates what type of build is being done.\n");  
    fprintf(stderr, "         M means VER_MAJOR++, VER_MINOR=0, VER_BASELEVEL=0, VER_PRODUCTBUILD=0\n");
    fprintf(stderr, "         I means VER_MINOR++, VER_BASELEVEL=0, VER_PRODUCTBUILD=0\n");
    fprintf(stderr, "         A means VER_BASELEVEL=1, VER_PRODUCTBUILD=0, or VER_PRODUCTBUILD++\n");
    fprintf(stderr, "         B means VER_BASELEVEL=2, VER_PRODUCTBUILD=0, or VER_PRODUCTBUILD++\n");
    fprintf(stderr, "          for A, B VER_PRODUCTBUILD++ if VER_BASELEVEL same since last build.\n");

    fprintf(stderr, "     -v  version sets the VER_MAJOR version to the indicated value.\n");

    fprintf(stderr, "     -i  always increment version.\n");

    fprintf(stderr, "     -a  always modify the version file, even if it is read-only\n");
    fprintf(stderr, "         The default is to not modify a read-only version file\n");

    fprintf(stderr, " The program sets the specified preprocessor constant in the\n");
    fprintf(stderr, " specified file to be one more than its current value.\n");
    fprintf(stderr, " If a list of file is specified after the program arguments, the version number "
                    " is\n incremented only if the version file is older than one of the targets.\n");
    return errno;
}

extern "C" {
    int getopt(int argc, char *argv[], char *opstring);
    extern char *optarg;
    extern int   optind;
    extern int   opterr;
}

int processOptions(int argc, char * argv[])
{
    char * optString = "ainrhf:s:v:b:";
    int opt;

    while ((opt = getopt(argc, argv, optString)) != EOF) {
        switch (opt) {

        case 'a':
        case 'A':
            override = 1;
            break;

        case 'b':
        case 'B':
            buildType = optarg;
            if((strcmp(buildType,"M") == 0) ||
               (strcmp(buildType,"I") == 0) ||
               (strcmp(buildType,"A") == 0) ||
               (strcmp(buildType,"B") == 0)) {
                break;
            } else {
                return 5;
            }

        case 'i':
        case 'I':
            noIncrement = 0;
            break;

        case 'n':
        case 'N':
            noIncrement = 1;
            break;

        case 'r':
        case 'R':
            report = 1;
            break;

        case 'f':
        case 'F':
            filename = optarg;
            break;

        case 'v':
        case 'V':
            version = atoi(optarg);
            break;

        case 'h':
        case 'H':
            (void) usage(argv[0], NULL, 0);
            return 2;

        default:
            (void) usage(argv[0], "unexpected option", 0);
            return 3;
        }
    }
    return 0;
}

//
// return 0 if the version file should be incremented
// return 1 if the version file should NOT be incremented
//
int testDates(int fileCount, char ** fileList)
{
    FILETIME versionTime;
    FILETIME fileTime;
    BOOL result;
    LONG difference;
    HANDLE whandle = CreateFile(filename, 
                        GENERIC_READ, 
                        FILE_SHARE_READ, 
                        NULL,
                        OPEN_EXISTING,
                        FILE_ATTRIBUTE_NORMAL,
                        NULL);

    if (whandle == INVALID_HANDLE_VALUE) {
        //
        // oops.
        //
        if (report) {

            printf("no version file to test dates\n");
        }
        return 0;
    }
    //
    // ok get the last modified time of the version file
    //
    result = GetFileTime( whandle,                 // handle to the file
                      NULL,    // address of creation time
                      NULL,  // address of last access time
                      &versionTime);   // address of last write time
    if (!result) {
        //
        // oh well
        //
        return 0;
    }
    //
    // now close the handle
    //
    CloseHandle(whandle);
    //
    // now open each file in filelist and
    // get the last modified time for the file
    //
    while (fileCount > 0) {

        if (fileList[0]) {

            if (report) {

                printf("checking timestamp on %s\n", fileList[0]);
            }

            whandle = CreateFile(fileList[0], 
                            GENERIC_READ, 
                            FILE_SHARE_READ, 
                            NULL,
                            OPEN_EXISTING,
                            FILE_ATTRIBUTE_NORMAL,
                            NULL);

            if (whandle == INVALID_HANDLE_VALUE) {
                //
                // oops.
                //
                fprintf(stderr, "Can't open source file %s error %d\n", 
                    fileList[0], GetLastError());
                return 0;
            }
            //
            // get this files mod time
            //
            result = GetFileTime( whandle,                 // handle to the file
                                  NULL,    // address of creation time
                                  NULL,  // address of last access time
                                  &fileTime);   // address of last write time
            if (!result) {
                //
                // gosh
                //
                fprintf(stderr, "Can't get time for file %s error %d\n", 
                    fileList[0], GetLastError());
                return 0;
            }
            //
            // close the handle
            //
            CloseHandle(whandle);
            //
            // is it greater than the version mod time?
            //
            difference =  CompareFileTime(&versionTime, &fileTime);
            switch (difference) {
            default:
                //
                // huh?
                //
            case -1:
            case 0:
                //
                // version is earlier than file
                //
                if (report) {

                    printf("version file older than %s\n", fileList[0]);
                }
                return 0;

            case 1:
                //
                // version is later than file
                //
                break;
            }
           
        } else {
            //
            // no fileList?
            //
            fprintf(stderr, "no file\n");
            return 0;
        } 
        fileCount--;
        fileList++;
    }
    //
    // if we get here then no files were newer than the
    // version file.
    //
    return 1;
}

int parseFile(FILE * handle, int& Major, int& Minor, int& Baselevel, int& Generation, char* Platform)
{
    char line[1024];
    const char * delim = " \t";
    char * word;
    int  lineCount = 0;

    //
    // The file must be as follows:
    //
    // version build number file
    // This file is automatically created. DO NOT EDIT IT.
    //
    // #define VER_MAJOR dddd
    // #define VER_MINOR dddd
    // #define VER_BASELEVEL 0 - minor, 1 - alpha  2 - beta
    // #define VER_PRODUCTBUILD dddd
    // #define VER_PLATFORM "string"
    //
    do {
        if (fgets(line, 1023, handle) == NULL) {
            return 7;
        }

        switch(lineCount) {

            case 0:
            case 1:
            case 2:
            case 3: 
                {
                    if (line[0] == '/') {
                        if (line[1] == '/') {
                            //
                            // skip comments
                            //
                            break;

                        } else {
                            //
                            // bogus file contents
                            //
                            return 1;
                        }
                    }
                }
                break;

            case 4: //VER_MAJOR
                {
                    word = strtok(line, delim);
                    if (word == NULL) {
                        return 2;
                    }
                    if (strcmp(word, "#define")) {
                        return 3;
                    }
                    word = strtok(NULL, delim);
                    if (word == NULL) {
                        return 4;
                    }
                    if (strcmp(word, "VER_MAJOR")) {
                        return 5;
                    }
                    word = strtok(NULL, delim);
                    if (word == NULL) {
                        return 6;
                    }
                    //
                    // word is the current value;
                    //
                    Major = atoi(word);
                }
                break;

            case 5: //VER_MINOR
                {
                    word = strtok(line, delim);
                    if (word == NULL) {
                        return 2;
                    }
                    if (strcmp(word, "#define")) {
                        return 3;
                    }
                    word = strtok(NULL, delim);
                    if (word == NULL) {
                        return 4;
                    }
                    if (strcmp(word, "VER_MINOR")) {
                        return 5;
                    }
                    word = strtok(NULL, delim);
                    if (word == NULL) {
                        return 6;
                    }
                    //
                    // word is the current value;
                    //
                    Minor = atoi(word);
                }
                break;

            case 6: //VER_BASELEVEL
                {
                    word = strtok(line, delim);
                    if (word == NULL) {
                        return 2;
                    }
                    if (strcmp(word, "#define")) {
                        return 3;
                    }
                    word = strtok(NULL, delim);
                    if (word == NULL) {
                        return 4;
                    }
                    if (strcmp(word, "VER_BASELEVEL")) {
                        return 5;
                    }
                    word = strtok(NULL, delim);
                    if (word == NULL) {
                        return 6;
                    }
                    //
                    // word is the current value;
                    //
                    Baselevel = atoi(word);
                }
                break;

            case 7: //VER_PRODUCTBUILD
                {
                    word = strtok(line, delim);
                    if (word == NULL) {
                        return 2;
                    }
                    if (strcmp(word, "#define")) {
                        return 3;
                    }
                    word = strtok(NULL, delim);
                    if (word == NULL) {
                        return 4;
                    }
                    if (strcmp(word, "VER_PRODUCTBUILD")) {
                        return 5;
                    }
                    word = strtok(NULL, delim);
                    if (word == NULL) {
                        return 6;
                    }
                    //
                    // word is the current value;
                    //
                    Generation = atoi(word);
                }
                break;

            case 8: //VER_PLATFORM
                {
                    word = strtok(line, delim);
                    if (word == NULL) {
                        return 2;
                    }
                    if (strcmp(word, "#define")) {
                        return 3;
                    }
                    word = strtok(NULL, delim);
                    if (word == NULL) {
                        return 4;
                    }
                    if (strcmp(word, "VER_PLATFORM")) {
                        return 5;
                    }
                    word = strtok(NULL, delim);
                    if (word == NULL) {
                        return 6;
                    }
                    //
                    // word is the current value;
                    //
                    strncpy(Platform,word,min(PLATFORM_MAX_SIZE-1,strlen(word)));
                }
                return 0;


            default:
                return 1;  // Bogus
        }

        lineCount++;

    } while(1);

}

int _cdecl _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

    memset(platform,0,sizeof(platform));


	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
        fprintf(stderr, "Fatal Error: MFC initialization failed\n");
		nRetCode = 1;
	}
	else
	{
        FILE * handle;
        char** fileList;
        int fileCount;
   

        int error = processOptions(argc, argv);

        if (error) {

            return error;
        }
        //
        // see if there are non option arguments left, if there are
        // these are treated as a list of files, each of which should be
        // tested to see if it is newer than version.h. Version.h is
        // incremented only if one of the specified files is newer
        //
        if (optind < argc) {
            fileList = &argv[optind];
            fileCount = argc - optind;

        } else {
            fileList = NULL;
            fileCount = 0;
        }

        if (version == 0) {
            //
            // read the current file for the version
            //
   
            if ((handle = fopen(filename, "r")) != NULL) {
                error = parseFile(handle, major, minor, baselevel, generation, platform);
                fclose(handle);
                if (error) {
                    fprintf(stderr, "version file corrupt (error %d) delete it and try again\n",
                        error);
                    return error;
                }
            
            }

            if (report) {
                printf("read Major number %d from symbol %s in file %s\n", 
                        major, "VER_MAJOR", filename);
                printf("read Minor number %d from symbol %s in file %s\n", 
                        minor, "VER_MINOR", filename);
                printf("read Baselevel number %d from symbol %s in file %s\n", 
                        baselevel, "VER_BASELEVEL", filename);
                printf("read Generation number %d from symbol %s in file %s\n", 
                        generation, "VER_PRODUCTBUILD", filename);
                printf("read Platform %s from symbol %s in file %s\n", 
                        platform, "VER_PLATFORM", filename);
            }

        } else if (report) {

            printf("setting version explicitly to %d for symbol %s in filename %s\n",
                    version, "VER_MAJOR", filename);
            major = version;
            minor = 0;
            baselevel = 0;
            generation = 0;

        }

        if (noIncrement) {

            printf("Major number %d from symbol %s\n", 
                    major, "VER_MAJOR");
            printf("Minor number %d from symbol %s\n", 
                    minor, "VER_MINOR");
            printf("Baselevel number %d from symbol %s\n", 
                    baselevel, "VER_BASELEVEL");
            printf("Generation number %d from symbol %s\n", 
                    generation, "VER_PRODUCTBUILD");
            printf("Platform %s from symbol %s\n", 
                    platform, "VER_PRODUCTBUILD");
            return 0;
        }

    
        if (fileCount) {

            noIncrement = testDates(fileCount, fileList);

        }

        if (noIncrement) {

            if (report) {

                printf("no source files newer than version file\n");
            }

            return 0;
        }

        //
        // Update the Platform
        //

        DWORD dwResult = ExpandEnvironmentStrings( "%DDK_TARGET_OS%", platform, 
                                            sizeof(platform)-1); 

        if(!dwResult) {

            error = GetLastError();

            fprintf(stderr, "Unable to get DDK_TARGET_OS environment variable. %x\n",
                error);
            return error;
        }

        //
        // create the new version file
        //
        handle = fopen(filename, "w");
    
        int originalMode = -1;

        if (handle == NULL) {

        
            //
            // ok - the version file is read only
            //
            if (override) {
                //
                // 2 == write
                // 4 == read
                // 6 == read/write
                //
                originalMode = 0;

                if (_access(filename, 4) == 0) {

                    originalMode |= _S_IREAD;

                }
                if (_access(filename, 2) == 0) {

                    originalMode |= _S_IWRITE;

                }

                if (-1 == _chmod( filename, _S_IREAD | _S_IWRITE )) {
                    //
                    // gak we can't change the file mode.
                    //
                    if (report) {
                        printf("can't change mode on version file, no increment\n");
                    }

                    return 0;
                }
                handle = fopen(filename, "w");            
            
            } 
        }
        if (handle == NULL) {
       
    
            if (report) {
                    printf("read-only version file (ok)\n");
                }

            return 0;
        }

        if(version) {

            major = version;
            minor = 0;
            baselevel = 0;
            generation = 0;

        } else {

            if(strcmp(buildType,"M") == 0) {

                if(baselevel != 0) {
                    major++;
                    minor = 0;
                    baselevel = 0;
                    generation = 0;
                } else {
                    generation++;
                }


            } else if(strcmp(buildType,"I") == 0) {

                if(baselevel != 1) {
                    minor++;
                    generation = 0;
                    baselevel = 1;
                } else {
                    generation++;
                }

            } else if(strcmp(buildType,"A") == 0) {

                if(baselevel != 2) {
                    generation = 0;
                    baselevel = 2;
                } else {
                    generation++;
                }

            } else if(strcmp(buildType,"B") == 0) {

                if(baselevel != 3) {
                    generation = 0;
                    baselevel = 3;
                } else {
                    generation++;
                }

            } else {

                generation++;

            }

        }
        //
        // Figure out what we're updating.
        //
        fprintf(handle, "//\n");
        fprintf(handle, "// version build number file\n");
        fprintf(handle, "// This file is automatically created. DO NOT EDIT IT.\n");
        fprintf(handle, "//\n");
        fprintf(handle, "#define VER_MAJOR %d\n", major);
        fprintf(handle, "#define VER_MINOR %d\n", minor);
        fprintf(handle, "#define VER_BASELEVEL %d\n", baselevel);
        fprintf(handle, "#define VER_PRODUCTBUILD %d\n", generation);
        fprintf(handle, "#define VER_PLATFORM %c%s%c\n",'"',platform,'"');

        fclose(handle);

        if (originalMode != -1) {
            //
            // perhaps we should reset it?
            // perhaps we should leave it be so
            // that the source control system will
            // detect that a read-only (as in checked-in)
            // file has been modified.
            //
        }

        if (report) {
            printf("new Major number %d from symbol %s in file %s\n", 
                    major, "VER_MAJOR", filename);
            printf("new Minor number %d from symbol %s in file %s\n", 
                    minor, "VER_MINOR", filename);
            printf("new Baselevel number %d from symbol %s in file %s\n", 
                    baselevel, "VER_BASELEVEL", filename);
            printf("new Product Build number %d from symbol %s in file %s\n", 
                    generation, "VER_PRODUCTBUILD", filename);
            printf("new Platform %s from symbol %s in file %s\n", 
                    platform, "VER_PLATFORM", filename);
        }
	}

	return nRetCode;
}


