///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 1995 - 2004 OSR Open Systems Resources, Inc.
//    All Rights Reserved         
//
//    This sofware is supplied for instructional purposes only.
//
//    OSR Open Systems Resources, Inc. (OSR) expressly disclaims any warranty
//    for this software.  THIS SOFTWARE IS PROVIDED  "AS IS" WITHOUT WARRANTY
//    OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION,
//    THE IMPLIED WARRANTIES OF MECHANTABILITY OR FITNESS FOR A PARTICULAR
//    PURPOSE.  THE ENTIRE RISK ARISING FROM THE USE OF THIS SOFTWARE REMAINS
//    WITH YOU.  OSR's entire liability and your exclusive remedy shall not
//    exceed the price paid for this material.  In no event shall OSR or its
//    suppliers be liable for any damages whatsoever (including, without
//    limitation, damages for loss of business profit, business interruption,
//    loss of business information, or any other pecuniary loss) arising out
//    of the use or inability to use this software, even if OSR has been
//    advised of the possibility of such damages.  Because some states/
//    jurisdictions do not allow the exclusion or limitation of liability for
//    consequential or incidental damages, the above limitation may not apply
//    to you.
//
//    OSR Open Systems Resources, Inc.
//    105 Route 101A Suite 19
//    Amherst, NH 03031  (603) 595-6500 FAX: (603) 595-6503
//    email bugs to: bugs@osr.com
//
//
//    MODULE:
//
//        Template.C -- Template Drive
//
//    ABSTRACT:
//
//      This file contains a device driver that does absolutely nothing.
//
//    AUTHOR(S):
//
//        OSR Open Systems Resources, Inc.
// 
//    REVISION:   
//
///////////////////////////////////////////////////////////////////////////////

#include <ntddk.h>



//
// Forward Declarations
//
// The following function appear in this file in the following order:
//
NTSTATUS DriverEntry(PDRIVER_OBJECT DriverObj, PUNICODE_STRING RegistryPath);
VOID     TemplateUnload(PDRIVER_OBJECT DriverObject);
NTSTATUS TemplateCreateClose(PDEVICE_OBJECT DeviceObject, PIRP Irp);




#define DRIVER_NAME "OSR Template"                      

//
// The following pragma allows the DriverEntry code to be discarded once
// initialization is completed
//
#pragma alloc_text(INIT,DriverEntry)


///////////////////////////////////////////////////////////////////////////////
//
//  DriverEntry
//
//    This routine is called by NT when the driver is first loaded.  It is the
//    responsibility of this routine to find it's device and create whatever
//    device objects it needs.
//
//  INPUTS:
//
//      DriverObj - Address of the DRIVER_OBJECT created by NT for this driver.
//
//      RegistryPath - UNICODE_STRING which represents this drivers KEY in the
//                   Registry.  
//
//  OUTPUTS:
//
//      None.
//
//  RETURNS:
//
//      STATUS_SUCCESS. Otherwise an error indicating why the driver could not
//                    Load.
//
//  IRQL:
//
//    This routine is called at IRQL_PASSIVE_LEVEL.
//
//  NOTES:
//
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS
DriverEntry(PDRIVER_OBJECT DriverObject, PUNICODE_STRING RegistryPath)
{
    NTSTATUS code;
    PDEVICE_OBJECT devObj;
    UNICODE_STRING devName;

#ifdef VERSION_NT4
    DbgPrint("\nOSR " DRIVER_NAME " Driver -NT4- Compiled %s %s\n",__DATE__, __TIME__);
#endif

#ifdef VERSION_W2K
    DbgPrint("\nOSR " DRIVER_NAME " Driver -W2K- Compiled %s %s\n",__DATE__, __TIME__);
#endif

#ifdef VERSION_WXP
    DbgPrint("\nOSR " DRIVER_NAME " Driver -WXP- Compiled %s %s\n",__DATE__, __TIME__);
#endif

#ifdef VERSION_NET
    DbgPrint("\nOSR " DRIVER_NAME " Driver -NET- Compiled %s %s\n",__DATE__, __TIME__);
#endif


    //
    // Establish dispatch entry points for the functions we support
    //

    //
    // Initialize the UNICODE device name.  This will be the "native NT" name
    // for our device.
    //
    RtlInitUnicodeString(&devName, L"\\Device\\Template");

    //
    // Ask the I/O Manager to create the device object.
    //

    code = IoCreateDevice(DriverObject,
                    0,
                    &devName,
                    FILE_DEVICE_UNKNOWN,
                    0,       
                    FALSE,
                    &devObj);


    if(!NT_SUCCESS(code))  {

#if DBG
        DbgPrint(DRIVER_NAME "IoCreateDevice failed.  Status = 0x%0x\n", code);
#endif
        return(STATUS_UNSUCCESSFUL);
    }    

    //
    // Note same function for create and close handling
    //
    DriverObject->MajorFunction[IRP_MJ_CREATE]         =  TemplateCreateClose;
    DriverObject->MajorFunction[IRP_MJ_CLOSE]          =  TemplateCreateClose;

    //
    // Unload function
    //
    DriverObject->DriverUnload = TemplateUnload;

#if DBG
    DbgPrint(DRIVER_NAME "DriverEntry: Leaving\n");
#endif

    return(STATUS_SUCCESS);
}


///////////////////////////////////////////////////////////////////////////////
//
//  TemplateUnload
//
//    This routine is our dynamic unload entry point.  We are called here when
//    the OS wants to unload our driver.  It is our responsibility to release
//    any global resources we allocated.  
//
//  INPUTS:
//
//      DriverObj - Address of our DRIVER_OBJECT.
//
//  OUTPUTS:
//
//      None.
//
//  RETURNS:
//
//      None.
//
//  IRQL:
//
//    This routine is called at IRQL_PASSIVE_LEVEL.
//
//  NOTES:
//
///////////////////////////////////////////////////////////////////////////////
VOID TemplateUnload(PDRIVER_OBJECT DriverObject)
{
#if DBG
    DbgPrint(DRIVER_NAME " Unload: Starting\n");
#endif

    IoDeleteDevice(DriverObject->DeviceObject);

    //
    // The device object should already be deleted (and GONE) at
    // this point 
    //
    ASSERT(!DriverObject->DeviceObject);

#if DBG
    DbgPrint(DRIVER_NAME " Unload: Leaving... \n");
#endif
}

///////////////////////////////////////////////////////////////////////////////
//
//  TemplateCreateClose
//
//    This is the dispatch entry point for processing both IRP_MJ_CREATE
//    and IRP_MJ_CLOSE functions.  Since this is a simple device driver,
//    there's really nothing to do here by complete the requests with success.
//
//
//  INPUTS:
//
//    DeviceObject - Address of the DEVICE_OBJECT for our device.
//  
//    Irp - Address of the IRP representing the IRP_MJ_CREATE or IRP_MJ_CLOSE
//          call.
//
//  OUTPUTS:
//
//      None.
//
//  RETURNS:
//
//    STATUS_SUCCESS.   We never fail this function.        
//
//  IRQL:
//
//    This routine is called at IRQL_PASSIVE_LEVEL.
//
//  NOTES:
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS TemplateCreateClose(PDEVICE_OBJECT DeviceObject, PIRP Irp)
{
#if DBG
    DbgPrint(DRIVER_NAME "CreateClose: Starting\n");
#endif

    //
    // Nothing much to do....
    //
    Irp->IoStatus.Status = STATUS_SUCCESS;
    Irp->IoStatus.Information = 0;

    IoCompleteRequest(Irp, IO_NO_INCREMENT);

#if DBG
    DbgPrint(DRIVER_NAME "CreateClose: Leaving\n");
#endif

    return(STATUS_SUCCESS);
}

